@@ -1,3 +1,11 @@
+from pkgutil import extend_path
+
+__path__ = extend_path(__path__, __name__)
+
+from .ydb_version import VERSION
+
+__version__ = VERSION
+
 from .credentials import *  # noqa
 from .driver import *  # noqa
 from .global_settings import *  # noqa
@@ -14,6 +22,9 @@ from .scripting import *  # noqa
 from .import_client import *  # noqa
 from .tracing import *  # noqa
 from .topic import *  # noqa
+from .draft import *  # noqa
+from .query import *  # noqa
+from .retries import *  # noqa
 
 try:
     import ydb.aio as aio  # noqa
