@@ -10,7 +10,15 @@ from ... import _utilities
 protobuf_version = Version(google.protobuf.__version__)
 
 # for compatible with arcadia
-if _utilities.check_module_exists("ydb.public.api"):
+if _utilities.check_module_exists("contrib.ydb.public.api"):
+    from contrib.ydb.public.api.grpc import *  # noqa
+
+    sys.modules["ydb._grpc.common"] = sys.modules["contrib.ydb.public.api.grpc"]
+
+    from contrib.ydb.public.api import protos
+
+    sys.modules["ydb._grpc.common.protos"] = sys.modules["contrib.ydb.public.api.protos"]
+elif _utilities.check_module_exists("ydb.public.api"):
     from ydb.public.api.grpc import *  # noqa
 
     sys.modules["ydb._grpc.common"] = sys.modules["ydb.public.api.grpc"]
@@ -28,7 +36,7 @@ else:
         from ydb._grpc.v3 import protos  # noqa
 
         sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb._grpc.v3.protos"]
-    else:
+    elif protobuf_version < Version("5.0"):
         from ydb._grpc.v4 import *  # noqa
 
         sys.modules["ydb._grpc.common"] = sys.modules["ydb._grpc.v4"]
@@ -36,3 +44,12 @@ else:
         from ydb._grpc.v4 import protos  # noqa
 
         sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb._grpc.v4.protos"]
+
+    else:
+        from ydb._grpc.v5 import *  # noqa
+
+        sys.modules["ydb._grpc.common"] = sys.modules["ydb._grpc.v5"]
+
+        from ydb._grpc.v5 import protos  # noqa
+
+        sys.modules["ydb._grpc.common.protos"] = sys.modules["ydb._grpc.v5.protos"]
