@@ -4,7 +4,7 @@ import copy
 import datetime
 import gzip
 import typing
-from collections import deque
+from collections import OrderedDict
 from dataclasses import dataclass
 from unittest import mock
 
@@ -14,7 +14,7 @@ from ydb import issues
 from . import datatypes, topic_reader_asyncio
 from .datatypes import PublicBatch, PublicMessage
 from .topic_reader import PublicReaderSettings
-from .topic_reader_asyncio import ReaderStream, ReaderReconnector
+from .topic_reader_asyncio import ReaderStream, ReaderReconnector, TopicReaderError
 from .._grpc.grpcwrapper.common_utils import SupportedDriverType, ServerStatus
 from .._grpc.grpcwrapper.ydb_topic import (
     StreamReadMessage,
@@ -36,6 +36,8 @@ if typing.TYPE_CHECKING:
 else:
     from .._grpc.common.protos import ydb_status_codes_pb2
 
+from .._constants import DEFAULT_INITIAL_RESPONSE_TIMEOUT
+
 
 @pytest.fixture(autouse=True)
 def handle_exceptions(event_loop):
@@ -52,9 +54,9 @@ def default_executor():
     executor.shutdown()
 
 
-def stub_partition_session():
-    return datatypes.PartitionSession(
-        id=0,
+def stub_partition_session(id: int = 0, ended: bool = False):
+    partition_session = datatypes.PartitionSession(
+        id=id,
         state=datatypes.PartitionSession.State.Active,
         topic_path="asd",
         partition_id=1,
@@ -63,6 +65,11 @@ def stub_partition_session():
         reader_stream_id=513,
     )
 
+    if ended:
+        partition_session.end()
+
+    return partition_session
+
 
 def stub_message(id: int):
     return PublicMessage(
@@ -73,7 +80,8 @@ def stub_message(id: int):
         offset=0,
         written_at=datetime.datetime(2023, 3, 18, 14, 15),
         producer_id="",
-        data=bytes(),
+        data=id,
+        metadata_items={},
         _partition_session=stub_partition_session(),
         _commit_start_offset=0,
         _commit_end_offset=1,
@@ -183,14 +191,14 @@ class TestReaderStream:
         yield stream_reader_started
 
         assert stream_reader_started._get_first_error() is None
-        await stream_reader_started.close()
+        await stream_reader_started.close(False)
 
     @pytest.fixture()
     async def stream_reader_finish_with_error(self, stream_reader_started: ReaderStream):
         yield stream_reader_started
 
         assert stream_reader_started._get_first_error() is not None
-        await stream_reader_started.close()
+        await stream_reader_started.close(False)
 
     @staticmethod
     def create_message(
@@ -207,16 +215,33 @@ class TestReaderStream:
             written_at=datetime.datetime(2023, 2, 3, 14, 16),
             producer_id="test-producer-id",
             data=bytes(),
+            metadata_items={},
             _partition_session=partition_session,
             _commit_start_offset=partition_session._next_message_start_commit_offset + offset_delta - 1,
             _commit_end_offset=partition_session._next_message_start_commit_offset + offset_delta,
         )
 
-    async def send_message(self, stream_reader, message: PublicMessage):
+    async def send_message(self, stream_reader, message: PublicMessage, new_batch=True):
+        await self.send_batch(stream_reader, [message], new_batch=new_batch)
+
+    async def send_batch(self, stream_reader, batch: typing.List[PublicMessage], new_batch=True):
+        if len(batch) == 0:
+            return
+
+        first_message = batch[0]
+        for message in batch:
+            assert message._partition_session is first_message._partition_session
+
+        partition_session_id = first_message._partition_session.id
+
         def batch_count():
             return len(stream_reader._message_batches)
 
+        def batch_size():
+            return len(stream_reader._message_batches[partition_session_id].messages)
+
         initial_batches = batch_count()
+        initial_batch_size = 0 if new_batch else batch_size()
 
         stream = stream_reader._stream  # type: StreamMock
         stream.from_server.put_nowait(
@@ -225,7 +250,7 @@ class TestReaderStream:
                 server_message=StreamReadMessage.ReadResponse(
                     partition_data=[
                         StreamReadMessage.ReadResponse.PartitionData(
-                            partition_session_id=message._partition_session.id,
+                            partition_session_id=first_message._partition_session.id,
                             batches=[
                                 StreamReadMessage.ReadResponse.Batch(
                                     message_data=[
@@ -234,14 +259,16 @@ class TestReaderStream:
                                             seq_no=message.seqno,
                                             created_at=message.created_at,
                                             data=message.data,
+                                            metadata_items={},
                                             uncompresed_size=len(message.data),
                                             message_group_id=message.message_group_id,
                                         )
+                                        for message in batch
                                     ],
-                                    producer_id=message.producer_id,
-                                    write_session_meta=message.session_metadata,
+                                    producer_id=first_message.producer_id,
+                                    write_session_meta=first_message.session_metadata,
                                     codec=Codec.CODEC_RAW,
-                                    written_at=message.written_at,
+                                    written_at=first_message.written_at,
                                 )
                             ],
                         )
@@ -250,7 +277,10 @@ class TestReaderStream:
                 ),
             )
         )
-        await wait_condition(lambda: batch_count() > initial_batches)
+        if new_batch:
+            await wait_condition(lambda: batch_count() > initial_batches)
+        else:
+            await wait_condition(lambda: batch_size() > initial_batch_size)
 
     async def test_unknown_error(self, stream, stream_reader_finish_with_error):
         class TestError(Exception):
@@ -361,9 +391,9 @@ class TestReaderStream:
         self, stream_reader_started: ReaderStream, partition_session
     ):
         waiter = partition_session.add_waiter(self.partition_session_committed_offset + 1)
-        await wait_for_fast(stream_reader_started.close())
+        await wait_for_fast(stream_reader_started.close(False))
 
-        with pytest.raises(topic_reader_asyncio.TopicReaderCommitToExpiredPartition):
+        with pytest.raises(topic_reader_asyncio.PublicTopicReaderPartitionExpiredError):
             waiter.future.result()
 
     async def test_flush(self, stream, stream_reader_started: ReaderStream, partition_session):
@@ -391,7 +421,7 @@ class TestReaderStream:
         # don't raises
         assert waiter.future.result() is None
 
-        await wait_for_fast(stream_reader_started.close())
+        await wait_for_fast(stream_reader_started.close(False))
 
     async def test_commit_ranges_for_received_messages(
         self, stream, stream_reader_started: ReaderStream, partition_session
@@ -401,15 +431,13 @@ class TestReaderStream:
         m2._commit_start_offset = m1.offset + 1
 
         await self.send_message(stream_reader_started, m1)
-        await self.send_message(stream_reader_started, m2)
+        await self.send_message(stream_reader_started, m2, new_batch=False)
 
         await stream_reader_started.wait_messages()
         received = stream_reader_started.receive_batch_nowait().messages
-        assert received == [m1]
+        assert received == [m1, m2]
 
-        await stream_reader_started.wait_messages()
-        received = stream_reader_started.receive_batch_nowait().messages
-        assert received == [m2]
+        await stream_reader_started.close(False)
 
     # noinspection PyTypeChecker
     @pytest.mark.parametrize(
@@ -427,6 +455,7 @@ class TestReaderStream:
                             written_at=datetime.datetime(2023, 3, 14, 15, 42),
                             producer_id="asd",
                             data=rb"123",
+                            metadata_items={},
                             _partition_session=None,
                             _commit_start_offset=5,
                             _commit_end_offset=15,
@@ -450,6 +479,7 @@ class TestReaderStream:
                             written_at=datetime.datetime(2023, 3, 14, 15, 42),
                             producer_id="asd",
                             data=gzip.compress(rb"123"),
+                            metadata_items={},
                             _partition_session=None,
                             _commit_start_offset=5,
                             _commit_end_offset=15,
@@ -472,6 +502,7 @@ class TestReaderStream:
                             offset=1,
                             written_at=datetime.datetime(2023, 3, 14, 15, 42),
                             producer_id="asd",
+                            metadata_items={},
                             data=rb"123",
                             _partition_session=None,
                             _commit_start_offset=5,
@@ -486,6 +517,7 @@ class TestReaderStream:
                             written_at=datetime.datetime(2023, 3, 14, 15, 42),
                             producer_id="asd",
                             data=rb"456",
+                            metadata_items={},
                             _partition_session=None,
                             _commit_start_offset=5,
                             _commit_end_offset=15,
@@ -509,6 +541,7 @@ class TestReaderStream:
                             written_at=datetime.datetime(2023, 3, 14, 15, 42),
                             producer_id="asd",
                             data=gzip.compress(rb"123"),
+                            metadata_items={},
                             _partition_session=None,
                             _commit_start_offset=5,
                             _commit_end_offset=15,
@@ -522,6 +555,7 @@ class TestReaderStream:
                             written_at=datetime.datetime(2023, 3, 14, 15, 42),
                             producer_id="asd",
                             data=gzip.compress(rb"456"),
+                            metadata_items={},
                             _partition_session=None,
                             _commit_start_offset=5,
                             _commit_end_offset=15,
@@ -574,10 +608,11 @@ class TestReaderStream:
                 StreamReadMessage.InitRequest.TopicReadSettings(
                     path="/local/test-topic",
                     partition_ids=[],
-                    max_lag_seconds=None,
+                    max_lag=None,
                     read_from=None,
                 )
             ],
+            auto_partitioning_support=False,
         )
         start_task = asyncio.create_task(reader._start(stream, init_message))
 
@@ -600,7 +635,7 @@ class TestReaderStream:
         )
 
         assert reader._session_id == "test"
-        await reader.close()
+        await reader.close(False)
 
     async def test_start_partition(
         self,
@@ -719,6 +754,101 @@ class TestReaderStream:
         with pytest.raises(asyncio.QueueEmpty):
             stream.from_client.get_nowait()
 
+    async def test_end_partition_session(self, stream, stream_reader, partition_session):
+        def session_count():
+            return len(stream_reader._partition_sessions)
+
+        initial_session_count = session_count()
+
+        stream.from_server.put_nowait(
+            StreamReadMessage.FromServer(
+                server_status=ServerStatus(ydb_status_codes_pb2.StatusIds.SUCCESS, []),
+                server_message=StreamReadMessage.EndPartitionSession(
+                    partition_session_id=partition_session.id,
+                    adjacent_partition_ids=[],
+                    child_partition_ids=[20, 30],
+                ),
+            )
+        )
+
+        await asyncio.sleep(0)  # wait next loop
+        with pytest.raises(asyncio.QueueEmpty):
+            stream.from_client.get_nowait()
+
+        assert session_count() == initial_session_count
+        assert partition_session.id in stream_reader._partition_sessions
+        assert partition_session.ended
+
+    @pytest.mark.parametrize(
+        "graceful",
+        (
+            [True],
+            [False],
+        ),
+    )
+    async def test_free_buffer_after_partition_stop(self, stream, stream_reader, partition_session, graceful):
+        initial_buffer_size = stream_reader._buffer_size_bytes
+        message_size = initial_buffer_size - 1
+
+        t = datetime.datetime.now(datetime.timezone.utc)
+
+        stream.from_server.put_nowait(
+            StreamReadMessage.FromServer(
+                server_status=ServerStatus(issues.StatusCode.SUCCESS, []),
+                server_message=StreamReadMessage.ReadResponse(
+                    bytes_size=message_size,
+                    partition_data=[
+                        StreamReadMessage.ReadResponse.PartitionData(
+                            partition_session_id=partition_session.id,
+                            batches=[
+                                StreamReadMessage.ReadResponse.Batch(
+                                    message_data=[
+                                        StreamReadMessage.ReadResponse.MessageData(
+                                            partition_session.committed_offset + 1,
+                                            seq_no=123,
+                                            created_at=t,
+                                            data=bytes(),
+                                            metadata_items={},
+                                            uncompresed_size=message_size,
+                                            message_group_id="test-message-group",
+                                        )
+                                    ],
+                                    producer_id="asd",
+                                    write_session_meta={},
+                                    codec=Codec.CODEC_RAW,
+                                    written_at=t,
+                                )
+                            ],
+                        )
+                    ],
+                ),
+            )
+        )
+
+        def message_received():
+            return len(stream_reader._message_batches) > 0
+
+        await wait_condition(message_received)
+
+        assert stream_reader._buffer_size_bytes == initial_buffer_size - message_size
+
+        stream.from_server.put_nowait(
+            StreamReadMessage.FromServer(
+                server_status=ServerStatus(issues.StatusCode.SUCCESS, []),
+                server_message=StreamReadMessage.StopPartitionSessionRequest(
+                    partition_session_id=partition_session.id,
+                    graceful=graceful,
+                    committed_offset=partition_session.committed_offset,
+                ),
+            )
+        )
+
+        await wait_condition(lambda: partition_session.closed)
+
+        batch = stream_reader.receive_batch_nowait()
+        assert not batch.alive
+        assert stream_reader._buffer_size_bytes == initial_buffer_size
+
     async def test_receive_message_from_server(
         self,
         stream_reader,
@@ -759,6 +889,7 @@ class TestReaderStream:
                                             created_at=created_at,
                                             data=data,
                                             uncompresed_size=len(data),
+                                            metadata_items={},
                                             message_group_id=message_group_id,
                                         )
                                     ],
@@ -778,7 +909,7 @@ class TestReaderStream:
 
         assert stream_reader._buffer_size_bytes == initial_buffer_size - bytes_size
 
-        last_batch = stream_reader._message_batches[-1]
+        _, last_batch = stream_reader._message_batches.popitem()
         assert last_batch == PublicBatch(
             messages=[
                 PublicMessage(
@@ -790,6 +921,7 @@ class TestReaderStream:
                     written_at=written_at,
                     producer_id=producer_id,
                     data=data,
+                    metadata_items={},
                     _partition_session=partition_session,
                     _commit_start_offset=expected_message_offset,
                     _commit_end_offset=expected_message_offset + 1,
@@ -836,6 +968,7 @@ class TestReaderStream:
                                         seq_no=3,
                                         created_at=created_at,
                                         data=data,
+                                        metadata_items={},
                                         uncompresed_size=len(data),
                                         message_group_id=message_group_id,
                                     )
@@ -857,6 +990,7 @@ class TestReaderStream:
                                         seq_no=2,
                                         created_at=created_at2,
                                         data=data,
+                                        metadata_items={},
                                         uncompresed_size=len(data),
                                         message_group_id=message_group_id,
                                     )
@@ -873,6 +1007,7 @@ class TestReaderStream:
                                         seq_no=3,
                                         created_at=created_at3,
                                         data=data2,
+                                        metadata_items={},
                                         uncompresed_size=len(data2),
                                         message_group_id=message_group_id,
                                     ),
@@ -881,6 +1016,7 @@ class TestReaderStream:
                                         seq_no=5,
                                         created_at=created_at4,
                                         data=data,
+                                        metadata_items={},
                                         uncompresed_size=len(data),
                                         message_group_id=message_group_id2,
                                     ),
@@ -911,6 +1047,7 @@ class TestReaderStream:
                     written_at=written_at,
                     producer_id=producer_id,
                     data=data,
+                    metadata_items={},
                     _partition_session=partition_session,
                     _commit_start_offset=partition1_mess1_expected_offset,
                     _commit_end_offset=partition1_mess1_expected_offset + 1,
@@ -931,6 +1068,7 @@ class TestReaderStream:
                     written_at=written_at2,
                     producer_id=producer_id,
                     data=data,
+                    metadata_items={},
                     _partition_session=second_partition_session,
                     _commit_start_offset=partition2_mess1_expected_offset,
                     _commit_end_offset=partition2_mess1_expected_offset + 1,
@@ -951,6 +1089,7 @@ class TestReaderStream:
                     written_at=written_at2,
                     producer_id=producer_id2,
                     data=data2,
+                    metadata_items={},
                     _partition_session=second_partition_session,
                     _commit_start_offset=partition2_mess2_expected_offset,
                     _commit_end_offset=partition2_mess2_expected_offset + 1,
@@ -964,6 +1103,7 @@ class TestReaderStream:
                     written_at=written_at2,
                     producer_id=producer_id,
                     data=data,
+                    metadata_items={},
                     _partition_session=second_partition_session,
                     _commit_start_offset=partition2_mess3_expected_offset,
                     _commit_end_offset=partition2_mess3_expected_offset + 1,
@@ -977,74 +1117,74 @@ class TestReaderStream:
     @pytest.mark.parametrize(
         "batches_before,expected_message,batches_after",
         [
-            ([], None, []),
+            ({}, None, {}),
             (
-                [
-                    PublicBatch(
+                {
+                    0: PublicBatch(
                         messages=[stub_message(1)],
                         _partition_session=stub_partition_session(),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     )
-                ],
+                },
                 stub_message(1),
-                [],
+                {},
             ),
             (
-                [
-                    PublicBatch(
+                {
+                    0: PublicBatch(
                         messages=[stub_message(1), stub_message(2)],
                         _partition_session=stub_partition_session(),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     ),
-                    PublicBatch(
+                    1: PublicBatch(
                         messages=[stub_message(3), stub_message(4)],
-                        _partition_session=stub_partition_session(),
+                        _partition_session=stub_partition_session(1),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     ),
-                ],
+                },
                 stub_message(1),
-                [
-                    PublicBatch(
+                {
+                    0: PublicBatch(
                         messages=[stub_message(2)],
                         _partition_session=stub_partition_session(),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     ),
-                    PublicBatch(
+                    1: PublicBatch(
                         messages=[stub_message(3), stub_message(4)],
-                        _partition_session=stub_partition_session(),
+                        _partition_session=stub_partition_session(1),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     ),
-                ],
+                },
             ),
             (
-                [
-                    PublicBatch(
+                {
+                    0: PublicBatch(
                         messages=[stub_message(1)],
                         _partition_session=stub_partition_session(),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     ),
-                    PublicBatch(
+                    1: PublicBatch(
                         messages=[stub_message(2), stub_message(3)],
-                        _partition_session=stub_partition_session(),
+                        _partition_session=stub_partition_session(1),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     ),
-                ],
+                },
                 stub_message(1),
-                [
-                    PublicBatch(
+                {
+                    1: PublicBatch(
                         messages=[stub_message(2), stub_message(3)],
-                        _partition_session=stub_partition_session(),
+                        _partition_session=stub_partition_session(1),
                         _bytes_size=0,
                         _codec=Codec.CODEC_RAW,
                     )
-                ],
+                },
             ),
         ],
     )
@@ -1055,47 +1195,236 @@ class TestReaderStream:
         expected_message: PublicMessage,
         batches_after: typing.List[datatypes.PublicBatch],
     ):
-        stream_reader._message_batches = deque(batches_before)
+        stream_reader._message_batches = OrderedDict(batches_before)
         mess = stream_reader.receive_message_nowait()
 
         assert mess == expected_message
-        assert list(stream_reader._message_batches) == batches_after
+        assert dict(stream_reader._message_batches) == batches_after
+
+    @pytest.mark.parametrize(
+        "batches,expected_order",
+        [
+            (
+                {
+                    0: PublicBatch(
+                        messages=[stub_message(1)],
+                        _partition_session=stub_partition_session(0, ended=True),
+                        _bytes_size=0,
+                        _codec=Codec.CODEC_RAW,
+                    )
+                },
+                [1],
+            ),
+            (
+                {
+                    0: PublicBatch(
+                        messages=[stub_message(1), stub_message(2)],
+                        _partition_session=stub_partition_session(0, ended=True),
+                        _bytes_size=0,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                    1: PublicBatch(
+                        messages=[stub_message(3), stub_message(4)],
+                        _partition_session=stub_partition_session(1),
+                        _bytes_size=0,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                },
+                [1, 2, 3, 4],
+            ),
+            (
+                {
+                    0: PublicBatch(
+                        messages=[stub_message(1), stub_message(2)],
+                        _partition_session=stub_partition_session(0),
+                        _bytes_size=0,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                    1: PublicBatch(
+                        messages=[stub_message(3), stub_message(4)],
+                        _partition_session=stub_partition_session(1, ended=True),
+                        _bytes_size=0,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                    2: PublicBatch(
+                        messages=[stub_message(5)],
+                        _partition_session=stub_partition_session(2),
+                        _bytes_size=0,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                },
+                [1, 3, 4, 5, 2],
+            ),
+        ],
+    )
+    async def test_read_message_autosplit_order(
+        self,
+        stream_reader,
+        batches: typing.Dict[int, datatypes.PublicBatch],
+        expected_order: typing.List[int],
+    ):
+        stream_reader._message_batches = OrderedDict(batches)
+
+        for id, batch in batches.items():
+            ps = batch._partition_session
+            stream_reader._partition_sessions[id] = ps
+
+        result = []
+        for _ in range(len(expected_order)):
+            mess = stream_reader.receive_message_nowait()
+            result.append(mess.data)
+
+        assert result == expected_order
+        assert stream_reader.receive_message_nowait() is None
+
+    @pytest.mark.parametrize(
+        "batches_before,max_messages,actual_messages,batches_after",
+        [
+            (
+                {
+                    0: PublicBatch(
+                        messages=[stub_message(1)],
+                        _partition_session=stub_partition_session(),
+                        _bytes_size=4,
+                        _codec=Codec.CODEC_RAW,
+                    )
+                },
+                None,
+                1,
+                {},
+            ),
+            (
+                {
+                    0: PublicBatch(
+                        messages=[stub_message(1), stub_message(2)],
+                        _partition_session=stub_partition_session(),
+                        _bytes_size=4,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                    1: PublicBatch(
+                        messages=[stub_message(3), stub_message(4)],
+                        _partition_session=stub_partition_session(1),
+                        _bytes_size=4,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                },
+                1,
+                1,
+                {
+                    1: PublicBatch(
+                        messages=[stub_message(3), stub_message(4)],
+                        _partition_session=stub_partition_session(1),
+                        _bytes_size=4,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                    0: PublicBatch(
+                        messages=[stub_message(2)],
+                        _partition_session=stub_partition_session(),
+                        _bytes_size=2,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                },
+            ),
+            (
+                {
+                    0: PublicBatch(
+                        messages=[stub_message(1)],
+                        _partition_session=stub_partition_session(),
+                        _bytes_size=4,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                    1: PublicBatch(
+                        messages=[stub_message(2), stub_message(3)],
+                        _partition_session=stub_partition_session(1),
+                        _bytes_size=4,
+                        _codec=Codec.CODEC_RAW,
+                    ),
+                },
+                100,
+                1,
+                {
+                    1: PublicBatch(
+                        messages=[stub_message(2), stub_message(3)],
+                        _partition_session=stub_partition_session(1),
+                        _bytes_size=4,
+                        _codec=Codec.CODEC_RAW,
+                    )
+                },
+            ),
+        ],
+    )
+    async def test_read_batch_max_messages(
+        self,
+        stream_reader,
+        batches_before: typing.List[datatypes.PublicBatch],
+        max_messages: typing.Optional[int],
+        actual_messages: int,
+        batches_after: typing.List[datatypes.PublicBatch],
+    ):
+        stream_reader._message_batches = OrderedDict(batches_before)
+        batch = stream_reader.receive_batch_nowait(max_messages=max_messages)
+
+        assert len(batch.messages) == actual_messages
+        assert stream_reader._message_batches == OrderedDict(batches_after)
 
     async def test_receive_batch_nowait(self, stream, stream_reader, partition_session):
         assert stream_reader.receive_batch_nowait() is None
 
+        initial_buffer_size = stream_reader._buffer_size_bytes
+
         mess1 = self.create_message(partition_session, 1, 1)
         await self.send_message(stream_reader, mess1)
 
         mess2 = self.create_message(partition_session, 2, 1)
-        await self.send_message(stream_reader, mess2)
+        await self.send_message(stream_reader, mess2, new_batch=False)
 
-        initial_buffer_size = stream_reader._buffer_size_bytes
+        assert stream_reader._buffer_size_bytes == initial_buffer_size - 2 * self.default_batch_size
 
         received = stream_reader.receive_batch_nowait()
         assert received == PublicBatch(
-            messages=[mess1],
+            messages=[mess1, mess2],
             _partition_session=mess1._partition_session,
-            _bytes_size=self.default_batch_size,
+            _bytes_size=self.default_batch_size * 2,
             _codec=Codec.CODEC_RAW,
         )
 
-        received = stream_reader.receive_batch_nowait()
-        assert received == PublicBatch(
-            messages=[mess2],
-            _partition_session=mess2._partition_session,
-            _bytes_size=self.default_batch_size,
-            _codec=Codec.CODEC_RAW,
+        assert stream_reader._buffer_size_bytes == initial_buffer_size
+
+        assert (
+            StreamReadMessage.ReadRequest(self.default_batch_size * 2) == stream.from_client.get_nowait().client_message
         )
 
-        assert stream_reader._buffer_size_bytes == initial_buffer_size + 2 * self.default_batch_size
-
-        assert StreamReadMessage.ReadRequest(self.default_batch_size) == stream.from_client.get_nowait().client_message
-        assert StreamReadMessage.ReadRequest(self.default_batch_size) == stream.from_client.get_nowait().client_message
-
         with pytest.raises(asyncio.QueueEmpty):
             stream.from_client.get_nowait()
 
+    async def test_receive_message_nowait(self, stream, stream_reader, partition_session):
+        assert stream_reader.receive_batch_nowait() is None
+
+        initial_buffer_size = stream_reader._buffer_size_bytes
+
+        await self.send_batch(
+            stream_reader,
+            [
+                self.create_message(partition_session, 1, 1),
+                self.create_message(partition_session, 2, 1),
+            ],
+        )
+        await self.send_batch(
+            stream_reader,
+            [
+                self.create_message(partition_session, 10, 1),
+            ],
+            new_batch=False,
+        )
+
+        assert stream_reader._buffer_size_bytes == initial_buffer_size - 2 * self.default_batch_size
+
+        for expected_seqno in [1, 2, 10]:
+            mess = stream_reader.receive_message_nowait()
+            assert mess.seqno == expected_seqno
+
+        assert stream_reader._buffer_size_bytes == initial_buffer_size
+
     async def test_update_token(self, stream):
         settings = PublicReaderSettings(
             consumer="test-consumer",
@@ -1123,7 +1452,70 @@ class TestReaderStream:
         got = await wait_for_fast(stream.from_client.get())
         assert expected == got
 
-        await reader.close()
+        await reader.close(False)
+
+    async def test_read_unknown_message(self, stream, stream_reader, caplog):
+        class TestMessage:
+            pass
+
+        # noinspection PyTypeChecker
+        stream.from_server.put_nowait(
+            StreamReadMessage.FromServer(
+                server_status=ServerStatus(
+                    status=issues.StatusCode.SUCCESS,
+                    issues=[],
+                ),
+                server_message=TestMessage(),
+            )
+        )
+
+        def logged():
+            for rec in caplog.records:
+                if TestMessage.__name__ in rec.message:
+                    return True
+            return False
+
+        await wait_condition(logged)
+
+    async def test_init_timeout_parameter(self, stream, default_reader_settings):
+        """Test that ReaderStream._start calls stream.receive with timeout=10"""
+        reader = ReaderStream(self.default_reader_reconnector_id, default_reader_settings)
+        init_message = default_reader_settings._init_message()
+
+        # Mock stream.receive to check if timeout is passed
+        with mock.patch.object(stream, "receive") as mock_receive:
+            mock_receive.return_value = StreamReadMessage.FromServer(
+                server_status=ServerStatus(ydb_status_codes_pb2.StatusIds.SUCCESS, []),
+                server_message=StreamReadMessage.InitResponse(session_id="test_session"),
+            )
+
+            await reader._start(stream, init_message)
+
+            # Verify that receive was called with timeout
+            mock_receive.assert_called_with(timeout=DEFAULT_INITIAL_RESPONSE_TIMEOUT)
+
+        await reader.close(False)
+
+    async def test_init_timeout_behavior(self, stream, default_reader_settings):
+        """Test that ReaderStream._start raises TopicReaderError when receive times out"""
+        reader = ReaderStream(self.default_reader_reconnector_id, default_reader_settings)
+        init_message = default_reader_settings._init_message()
+
+        # Mock stream.receive to directly raise TimeoutError when called with timeout
+        async def timeout_receive(timeout=None):
+            if timeout == DEFAULT_INITIAL_RESPONSE_TIMEOUT:
+                raise asyncio.TimeoutError("Simulated timeout")
+            return StreamReadMessage.FromServer(
+                server_status=ServerStatus(ydb_status_codes_pb2.StatusIds.SUCCESS, []),
+                server_message=StreamReadMessage.InitResponse(session_id="test_session"),
+            )
+
+        with mock.patch.object(stream, "receive", side_effect=timeout_receive):
+            # Should raise TopicReaderError with timeout message
+            with pytest.raises(TopicReaderError, match="Timeout waiting for init response"):
+                await reader._start(stream, init_message)
+
+        await reader.close(False)
 
 
 @pytest.mark.asyncio
@@ -1135,15 +1527,21 @@ class TestReaderReconnector:
             raise test_error
 
         reader_stream_mock_with_error = mock.Mock(ReaderStream)
+        reader_stream_mock_with_error._id = 0
         reader_stream_mock_with_error.wait_error = mock.AsyncMock(side_effect=wait_error)
 
-        async def wait_messages():
+        async def wait_messages_with_error():
             raise test_error
 
-        reader_stream_mock_with_error.wait_messages = mock.AsyncMock(side_effect=wait_messages)
+        reader_stream_mock_with_error.wait_messages = mock.AsyncMock(side_effect=wait_messages_with_error)
+
+        async def wait_forever():
+            f = asyncio.Future()
+            await f
 
         reader_stream_with_messages = mock.Mock(ReaderStream)
-        reader_stream_with_messages.wait_error.return_value = asyncio.Future()
+        reader_stream_with_messages._id = 0
+        reader_stream_with_messages.wait_error = mock.AsyncMock(side_effect=wait_forever)
         reader_stream_with_messages.wait_messages.return_value = None
 
         stream_index = 0
