@@ -102,8 +102,8 @@ class MetadataUrlCredentials(AbstractExpiringTokenCredentials):
         super(MetadataUrlCredentials, self).__init__()
         assert aiohttp is not None, "Install aiohttp library to use metadata credentials provider"
         self._metadata_url = auth.DEFAULT_METADATA_URL if metadata_url is None else metadata_url
-        self._tp.submit(self._refresh)
         self.extra_error_message = "Check that metadata service configured properly and application deployed in VM or function at Yandex.Cloud."
+        self._tp.submit(self._refresh_token)
 
     async def _make_token_request(self):
         timeout = aiohttp.ClientTimeout(total=2)
