@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import logging
 import copy
+import typing
 from concurrent import futures
 import uuid
 import threading
@@ -25,6 +26,7 @@ YDB_TRACE_ID_HEADER = "x-ydb-trace-id"
 YDB_REQUEST_TYPE_HEADER = "x-ydb-request-type"
 
 _DEFAULT_MAX_GRPC_MESSAGE_SIZE = 64 * 10**6
+_DEFAULT_KEEPALIVE_TIMEOUT = 10000
 
 
 def _message_to_string(message):
@@ -61,21 +63,30 @@ def _log_request(rpc_state, request):
         logger.debug("%s: request = { %s }", rpc_state, _message_to_string(request))
 
 
-def _rpc_error_handler(rpc_state, rpc_error, on_disconnected=None):
+def _rpc_error_handler(
+    rpc_state,
+    rpc_error: typing.Union[grpc.RpcError, grpc.aio.AioRpcError, grpc.Call, grpc.aio.Call],
+    on_disconnected: typing.Callable[[], None] = None,
+    use_unavailable: bool = False,
+):
     """
     RPC call error handler, that translates gRPC error into YDB issue
     :param rpc_state: A state of rpc
     :param rpc_error: an underlying rpc error to handle
     :param on_disconnected: a handler to call on disconnected connection
     """
-    logger.info("%s: received error, %s", rpc_state, rpc_error)
-    if isinstance(rpc_error, grpc.Call):
+    logger.debug("%s: received error, %s", rpc_state, rpc_error)
+    if isinstance(rpc_error, (grpc.RpcError, grpc.aio.AioRpcError, grpc.Call, grpc.aio.Call)):
         if rpc_error.code() == grpc.StatusCode.UNAUTHENTICATED:
             return issues.Unauthenticated(rpc_error.details())
         elif rpc_error.code() == grpc.StatusCode.DEADLINE_EXCEEDED:
             return issues.DeadlineExceed("Deadline exceeded on request")
         elif rpc_error.code() == grpc.StatusCode.UNIMPLEMENTED:
             return issues.Unimplemented("Method or feature is not implemented on server!")
+        elif rpc_error.code() == grpc.StatusCode.CANCELLED:
+            return issues.Cancelled(rpc_error.details())
+        elif use_unavailable and rpc_error.code() == grpc.StatusCode.UNAVAILABLE:
+            return issues.Unavailable(rpc_error.details())
 
     logger.debug("%s: unhandled rpc error, disconnecting channel", rpc_state)
     if on_disconnected is not None:
@@ -84,6 +95,16 @@ def _rpc_error_handler(rpc_state, rpc_error, on_disconnected=None):
     return issues.ConnectionLost("Rpc error, reason %s" % str(rpc_error))
 
 
+def _is_disconnect_needed(error):
+    return isinstance(
+        error,
+        (
+            issues.ConnectionLost,
+            issues.Unavailable,
+        ),
+    )
+
+
 def _on_response_callback(rpc_state, call_state_unref, wrap_result=None, on_disconnected=None, wrap_args=()):
     """
     Callback to be executed on received RPC response
@@ -180,15 +201,18 @@ def _construct_channel_options(driver_config, endpoint_options=None):
             getattr(driver_config, "grpc_lb_policy_name", "round_robin"),
         ),
     ]
-    if driver_config.grpc_keep_alive_timeout is not None:
-        _default_connect_options.extend(
-            [
-                ("grpc.keepalive_time_ms", driver_config.grpc_keep_alive_timeout >> 3),
-                ("grpc.keepalive_timeout_ms", driver_config.grpc_keep_alive_timeout),
-                ("grpc.http2.max_pings_without_data", 0),
-                ("grpc.keepalive_permit_without_calls", 0),
-            ]
-        )
+    if driver_config.grpc_keep_alive_timeout is None:
+        driver_config.grpc_keep_alive_timeout = _DEFAULT_KEEPALIVE_TIMEOUT
+
+    _default_connect_options.extend(
+        [
+            ("grpc.keepalive_time_ms", driver_config.grpc_keep_alive_timeout >> 3),
+            ("grpc.keepalive_timeout_ms", driver_config.grpc_keep_alive_timeout),
+            ("grpc.http2.max_pings_without_data", 0),
+            ("grpc.keepalive_permit_without_calls", 0),
+        ]
+    )
+
     if endpoint_options is not None:
         if endpoint_options.ssl_target_name_override:
             _default_connect_options.append(
@@ -316,6 +340,33 @@ class EndpointKey(object):
         self.node_id = node_id
 
 
+class _SafeSyncIterator:
+    def __init__(self, resp, rpc_state, on_disconnected_callback):
+        self.resp = resp
+        self.it = resp.__iter__()
+        self.rpc_state = rpc_state
+        self.on_disconnected_callback = on_disconnected_callback
+
+    def cancel(self):
+        self.resp.cancel()
+        return self
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        try:
+            return self.it.__next__()
+        except grpc.RpcError as rpc_error:
+            ydb_error = _rpc_error_handler(self.rpc_state, rpc_error, use_unavailable=True)
+            if _is_disconnect_needed(ydb_error):
+                self.on_disconnected_callback()
+            raise ydb_error
+
+    def __getattr__(self, item):
+        return getattr(self.resp, item)
+
+
 class Connection(object):
     __slots__ = (
         "endpoint",
@@ -457,6 +508,11 @@ class Connection(object):
                 compression=getattr(settings, "compression", None),
             )
             _log_response(rpc_state, response)
+
+            if hasattr(response, "__iter__"):
+                # NOTE(vgvoleg): for stream results we should also be able to handle disconnects
+                response = _SafeSyncIterator(response, rpc_state, on_disconnected)
+
             return response if wrap_result is None else wrap_result(rpc_state, response, *wrap_args)
         except grpc.RpcError as rpc_error:
             raise _rpc_error_handler(rpc_state, rpc_error, on_disconnected)
@@ -490,7 +546,7 @@ class Connection(object):
         Closes the underlying gRPC channel
         :return: None
         """
-        logger.info("Closing channel for endpoint %s", self.endpoint)
+        logger.debug("Closing channel for endpoint %s", self.endpoint)
         with self.lock:
             self.closing = True
 
