@@ -1,10 +1,19 @@
 import enum
+import typing
 
 from . import _apis
 
 from . import settings_impl as s_impl
-from ydb._grpc.common.protos import ydb_import_pb2
-from ydb._grpc.common import ydb_import_v1_pb2_grpc
+
+# Workaround for good IDE and universal for runtime
+if typing.TYPE_CHECKING:
+    from ._grpc.v4.protos import ydb_import_pb2
+    from ._grpc.v4 import ydb_import_v1_pb2_grpc
+else:
+    from ._grpc.common.protos import ydb_import_pb2
+    from ._grpc.common import ydb_import_v1_pb2_grpc
+
+
 from . import operation
 
 _ImportFromS3 = "ImportFromS3"
@@ -24,7 +33,10 @@ class ImportProgress(enum.IntEnum):
 
 def _initialize_progresses():
     for key, value in ydb_import_pb2.ImportProgress.Progress.items():
-        _progresses[value] = getattr(ImportProgress, key[len("PROGRESS_") :])
+        try:
+            _progresses[value] = getattr(ImportProgress, key[len("PROGRESS_") :])
+        except AttributeError:
+            pass
 
 
 _initialize_progresses()
