@@ -5,11 +5,12 @@ import logging
 from concurrent import futures
 import collections
 import random
+import typing
 
 from . import connection as connection_impl, issues, resolver, _utilities, tracing
 from abc import abstractmethod
 
-from .connection import Connection
+from .connection import Connection, EndpointKey
 
 logger = logging.getLogger(__name__)
 
@@ -123,13 +124,13 @@ class ConnectionsCache(object):
             return subscription
 
     @tracing.with_trace()
-    def get(self, preferred_endpoint=None) -> Connection:
+    def get(self, preferred_endpoint: typing.Optional[EndpointKey] = None) -> Connection:
         with self.lock:
             if preferred_endpoint is not None and preferred_endpoint.node_id in self.connections_by_node_id:
                 return self.connections_by_node_id[preferred_endpoint.node_id]
 
             if preferred_endpoint is not None and preferred_endpoint.endpoint in self.connections:
-                return self.connections[preferred_endpoint]
+                return self.connections[preferred_endpoint.endpoint]
 
             for conn_lst in self.conn_lst_order:
                 try:
@@ -349,8 +350,21 @@ class ConnectionPool(IConnectionPool):
         self._store = ConnectionsCache(driver_config.use_all_nodes, driver_config.tracer)
         self.tracer = driver_config.tracer
         self._grpc_init = connection_impl.Connection(self._driver_config.endpoint, self._driver_config)
-        self._discovery_thread = Discovery(self._store, self._driver_config)
-        self._discovery_thread.start()
+
+        if driver_config.disable_discovery:
+            # If discovery is disabled, just add the initial endpoint to the store
+            ready_connection = connection_impl.Connection.ready_factory(
+                self._driver_config.endpoint,
+                self._driver_config,
+                ready_timeout=getattr(self._driver_config, "discovery_request_timeout", 10),
+            )
+            self._store.add(ready_connection)
+            self._discovery_thread = None
+        else:
+            # Start discovery thread as usual
+            self._discovery_thread = Discovery(self._store, self._driver_config)
+            self._discovery_thread.start()
+
         self._stopped = False
         self._stop_guard = threading.Lock()
 
@@ -366,9 +380,11 @@ class ConnectionPool(IConnectionPool):
                 return
 
             self._stopped = True
-            self._discovery_thread.stop()
+            if self._discovery_thread:
+                self._discovery_thread.stop()
         self._grpc_init.close()
-        self._discovery_thread.join(timeout)
+        if self._discovery_thread:
+            self._discovery_thread.join(timeout)
 
     def async_wait(self, fail_fast=False):
         """
@@ -403,7 +419,13 @@ class ConnectionPool(IConnectionPool):
         self._discovery_thread.notify_disconnected()
 
     def discovery_debug_details(self):
-        return self._discovery_thread.discovery_debug_details()
+        """
+        Returns debug string about last errors
+        :return: str
+        """
+        if self._discovery_thread:
+            return self._discovery_thread.discovery_debug_details()
+        return "Discovery is disabled, using only the initial endpoint"
 
     @tracing.with_trace()
     def __call__(
@@ -429,6 +451,9 @@ class ConnectionPool(IConnectionPool):
 
         :return: A result of computation
         """
+        if self._stopped:
+            raise issues.Error("Driver was stopped")
+
         tracing.trace(self.tracer, {"request": request, "stub": stub, "rpc_name": rpc_name})
         try:
             connection = self._store.get(preferred_endpoint)
@@ -445,7 +470,9 @@ class ConnectionPool(IConnectionPool):
             wrap_args,
             lambda: self._on_disconnected(connection),
         )
+
         tracing.trace(self.tracer, {"response": res}, trace_level=tracing.TraceLevel.DEBUG)
+
         return res
 
     @_utilities.wrap_async_call_exceptions
